﻿using System;
using System.Collections.Generic;
using System.Text;
using TaskScheduler;
using System.Reflection;
using System.IO;
using System.Runtime.InteropServices;

namespace Eq2FullscreenWindowizer
{
    internal static class AutoStartManager
    {
        private static string CurrentPath
        {
            get
            {
                Assembly easm = Assembly.GetEntryAssembly();
                FileInfo fi = new FileInfo(easm.Location);
                return fi.FullName;
            }
        }

        private static bool IsEquivalentToCurrentPath(string pathToCheck)
        {
            FileInfo fi = new FileInfo(pathToCheck);
            if (fi.FullName.Equals(CurrentPath, StringComparison.OrdinalIgnoreCase))
                return true;
            else
                return false;
        }

        private static string FindAutoStartTask(bool delete)
        {
            string username = System.Environment.UserDomainName + "\\" + System.Environment.UserName;

            TaskSchedulerClass tsvc = null;
            try
            {
                tsvc = new TaskSchedulerClass();
                tsvc.Connect();
                ITaskFolder rootFolder = tsvc.GetFolder("\\");
                foreach (IRegisteredTask ttask in rootFolder.GetTasks(0))
                {
                    bool isLogonTask = false;
                    foreach (ITrigger ttrigger in ttask.Definition.Triggers)
                    {
                        if (ttrigger.Type == _TASK_TRIGGER_TYPE2.TASK_TRIGGER_LOGON)
                        {
                            ILogonTrigger lt = (ILogonTrigger)ttrigger;
                            if (lt.UserId.Equals(username))
                            {
                                isLogonTask = true;
                                break;
                            }
                        }
                    }
                    if (!isLogonTask)
                        continue;

                    bool isPointingToCurrentExecutable = false;
                    foreach (IAction taction in ttask.Definition.Actions)
                    {
                        if (taction is IExecAction)
                        {
                            IExecAction xa = (IExecAction)taction;

                            if (IsEquivalentToCurrentPath(xa.Path))
                            {
                                isPointingToCurrentExecutable = true;
                                break;
                            }
                        }
                    }
                    if (!isPointingToCurrentExecutable)
                        continue;

                    if (delete)
                    {
                        rootFolder.DeleteTask(ttask.Name, 0);
                    }
                    return ttask.Name;
                }

                return null;
            }
            finally
            {
                if (tsvc != null)
                {
                    try
                    {
                        Marshal.ReleaseComObject(tsvc);
                    }
                    catch { }
                }
            }
        }

        internal static bool IsAutoStartConfigured
        {
            get
            {
                return (FindAutoStartTask(false) != null);
            }
            set
            {
                if (value != IsAutoStartConfigured)
                {
                    if (!value)
                    {
                        UnregisterAutoStartTask();
                    }
                    else
                    {
                        RegisterAutoStartTask();
                    }
                }
            }
        }

        private static void RegisterAutoStartTask()
        {
            TaskSchedulerClass tsvc = null;
            try
            {
                tsvc = new TaskSchedulerClass();
                tsvc.Connect();
                ITaskFolder rootFolder = tsvc.GetFolder("\\");

                ITaskDefinition newTask = tsvc.NewTask(0);
                IRegistrationInfo newTaskReg = newTask.RegistrationInfo;
                ITaskSettings newTaskSettings = newTask.Settings;

                string username = System.Environment.UserDomainName + "\\" +
                    System.Environment.UserName;

                newTaskReg.Description = "Automatically execute EQ2 Fullscreen Windowizer when the user \"" + username + "\" logs on.";
                newTaskReg.Author = "Eq2FullscreenWindowizer.exe";

                newTaskSettings.StartWhenAvailable = true;
                newTaskSettings.DisallowStartIfOnBatteries = false;
                newTaskSettings.StopIfGoingOnBatteries = false;
                newTaskSettings.IdleSettings.StopOnIdleEnd = false;

                newTaskSettings.ExecutionTimeLimit = "PT0S";

                ILogonTrigger lt = (ILogonTrigger)newTask.Triggers.Create(_TASK_TRIGGER_TYPE2.TASK_TRIGGER_LOGON);
                //lt.StartBoundary = "2010-01-01T00:00:00";
                //lt.EndBoundary = "2100-01-01T00:00:00";
                lt.Id = "LogonTriggerId";
                lt.UserId = username;

                IExecAction xa = (IExecAction)newTask.Actions.Create(_TASK_ACTION_TYPE.TASK_ACTION_EXEC);
                xa.Path = CurrentPath;
                xa.Arguments = "/autostart";
                xa.WorkingDirectory = Path.GetDirectoryName(CurrentPath);

                newTask.Principal.RunLevel = _TASK_RUNLEVEL.TASK_RUNLEVEL_HIGHEST;

                const int TASK_CREATE_OR_UPDATE = 6;
                rootFolder.RegisterTaskDefinition("EQ2FSW Autostart (" + Guid.NewGuid().ToString() + ")",
                    newTask, TASK_CREATE_OR_UPDATE,
                    Type.Missing, Type.Missing, _TASK_LOGON_TYPE.TASK_LOGON_INTERACTIVE_TOKEN);
            }
            finally
            {
                if (tsvc != null)
                {
                    try
                    {
                        Marshal.ReleaseComObject(tsvc);
                    }
                    catch { }
                }
            }
        }

        private static void UnregisterAutoStartTask()
        {
            FindAutoStartTask(true);
        }
    }
}
