﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
using System.Reflection;
using System.IO;

namespace Eq2FullscreenWindowizer
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            SetIcon();
        }

        private void SetIcon()
        {
            Icon ico = Program.GetApplicationIcon();
            if (ico != null)
                this.Icon = ico;
        }

        private enum WindowLongIndex : int
        {
            GWL_EXSTYLE = -20,
            GWL_STYLE = -16,
            GWL_WNDPROC = -4,
            GWL_HINSTANCE = -6,
            GWL_HWNDPARENT = -8,
            GWL_ID = -12,
            GWL_USERDATA = -21,
            DWL_DLGPROC = 4,
            DWL_MSGRESULT = 0,
            DWL_USER = 8
        }

        private enum WindowStyles : uint
        {
            WS_MAXIMIZEBOX = 0x10000, /* x */
            WS_MINIMIZEBOX = 0x20000, /* x */
            WS_THICKFRAME = 0x40000, /* x */
            WS_SYSMENU = 0x80000, /* x */
            WS_HSCROLL = 0x100000,
            WS_VSCROLL = 0x200000,
            WS_DLGFRAME = 0x400000, /* x */
            WS_BORDER = 0x800000, /* x */
            WS_MAXIMIZE = 0x1000000,
            WS_CLIPCHILDREN = 0x2000000,
            WS_CLIPSIBLINGS = 0x4000000,
            WS_DISABLED = 0x8000000,
            WS_VISIBLE = 0x10000000,
            WS_MINIMIZE = 0x20000000,
            WS_CHILD = 0x40000000,
            WS_POPUP = 0x80000000
        }

        private enum WindowStylesEx : uint
        {
            WS_EX_DLGMODALFRAME = 0x0001,
            WS_EX_NOPARENTNOTIFY = 0x0004,
            WS_EX_TOPMOST = 0x0008,
            WS_EX_ACCEPTFILES = 0x0010,
            WS_EX_TRANSPARENT = 0x0020,
            WS_EX_MDICHILD = 0x0040,
            WS_EX_TOOLWINDOW = 0x0080,
            WS_EX_WINDOWEDGE = 0x0100,
            WS_EX_CLIENTEDGE = 0x0200,
            WS_EX_CONTEXTHELP = 0x0400,
            WS_EX_RIGHT = 0x1000,
            WS_EX_RTLREADING = 0x2000,
            WS_EX_LEFTSCROLLBAR = 0x4000,
            WS_EX_CONTROLPARENT = 0x10000,
            WS_EX_STATICEDGE = 0x20000,
            WS_EX_APPWINDOW = 0x40000,
            WS_EX_LAYERED = 0x00080000,
            WS_EX_NOINHERITLAYOUT = 0x00100000, // Disable inheritence of mirroring by children
            WS_EX_LAYOUTRTL = 0x00400000, // Right to left mirroring
            WS_EX_COMPOSITED = 0x02000000,
            WS_EX_NOACTIVATE = 0x08000000
        }

        [Flags]
        private enum SetWindowPosFlags : uint
        {
            SWP_NOSIZE = 0x0001,
            SWP_NOMOVE = 0x0002,
            SWP_NOZORDER = 0x0004,
            SWP_NOREDRAW = 0x0008,
            SWP_NOACTIVATE = 0x0010,
            SWP_FRAMECHANGED = 0x0020,  /* The frame changed: send WM_NCCALCSIZE */
            SWP_SHOWWINDOW = 0x0040,
            SWP_HIDEWINDOW = 0x0080,
            SWP_NOCOPYBITS = 0x0100,
            SWP_NOOWNERZORDER = 0x0200,  /* Don't do owner Z ordering */
            SWP_NOSENDCHANGING = 0x0400  /* Don't send WM_WINDOWPOSCHANGING */
        }

        [DllImport("user32.dll")]
        static extern bool SetWindowPos(IntPtr hWnd, IntPtr hWndInsertAfter, int X, int Y, int cx, int cy, SetWindowPosFlags uFlags);

        [DllImport("user32.dll", SetLastError = true)]
        static extern int GetWindowLong(IntPtr hWnd, WindowLongIndex nIndex);

        private static WindowStyles GetWindowStyles(IntPtr hWnd)
        {
            return (WindowStyles)GetWindowLong(hWnd, WindowLongIndex.GWL_STYLE);
        }

        private static WindowStylesEx GetWindowStylesEx(IntPtr hWnd)
        {
            return (WindowStylesEx)GetWindowLong(hWnd, WindowLongIndex.GWL_EXSTYLE);
        }

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, WindowLongIndex nIndex, int dwNewLong);

        private static void SetWindowStyles(IntPtr hWnd, WindowStyles styles)
        {
            SetWindowLong(hWnd, WindowLongIndex.GWL_STYLE, (int)styles);
        }

        private static void SetWindowStylesEx(IntPtr hWnd, WindowStylesEx stylesEx)
        {
            SetWindowLong(hWnd, WindowLongIndex.GWL_EXSTYLE, (int)stylesEx);
        }

        private class CbScreen
        {
            public CbScreen(int idx, Screen scr)
            {
                m_idx = idx;
                m_screen = scr;
            }

            private readonly int m_idx;
            private readonly Screen m_screen;

            public Screen Screen
            {
                get { return m_screen; }
            }

            public override string ToString()
            {
                return m_idx.ToString() + ": " + m_screen.DeviceName;
            }
        }

        private ShellHook m_shook;

        private void Form1_Load(object sender, EventArgs e)
        {
            m_shook = new ShellHook();
            m_shook.WindowCreated += new EventHandler<HWndEventArgs>(m_shook_WindowCreated);

            int idx = 0;
            foreach (Screen scr in Screen.AllScreens)
            {
                idx++;
                comboBox1.Items.Add(new CbScreen(idx, scr));
            }

            cbAutoFit.Checked = PersistentSettings.AutoSize;
            comboBox1.SelectedIndex = PersistentSettings.ScreenIndex;

            cbStartAutomatically.Checked = AutoStartManager.IsAutoStartConfigured;
        }

        private void m_shook_WindowCreated(object sender, HWndEventArgs e)
        {
            //listBox1.Items.Add("Window created: " + NativeMethods.GetWindowText(e.Handle));
            if (cbAutoFit.Checked && IsEq2Window(e.Handle))
            {
                PerformWindowFit(e.Handle, ((CbScreen)comboBox1.SelectedItem).Screen);
            }
        }

        private bool IsEq2Window(IntPtr hWnd)
        {
            try
            {
                string winTitle = NativeMethods.GetWindowText(hWnd);
                return (winTitle.Contains("EverQuest II") && winTitle.Contains("SOEBuild"));
            }
            catch
            {
                return false;
            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            PersistentSettings.ScreenIndex = comboBox1.SelectedIndex;
        }

        private void button4_Click(object sender, EventArgs e)
        {
            foreach (IntPtr hWnd in NativeMethods.EnumWindows())
            {
                if (IsEq2Window(hWnd))
                {
                    PerformWindowFit(hWnd, ((CbScreen)comboBox1.SelectedItem).Screen);
                    break;
                }
            }
        }

        private void PerformWindowFit(IntPtr hWnd, Screen scr)
        {
            Rectangle r = scr.Bounds;

            WindowStyles ws = GetWindowStyles(hWnd);
            ws ^= WindowStyles.WS_MAXIMIZEBOX | WindowStyles.WS_MINIMIZEBOX | WindowStyles.WS_THICKFRAME |
                WindowStyles.WS_SYSMENU | WindowStyles.WS_DLGFRAME | WindowStyles.WS_BORDER;

            SetWindowStyles(hWnd, ws);
            SetWindowPos(hWnd, IntPtr.Zero, r.X, r.Y, r.Width, r.Height,
                /*SetWindowPosFlags.SWP_NOMOVE | SetWindowPosFlags.SWP_NOSIZE |*/ SetWindowPosFlags.SWP_NOZORDER |
                SetWindowPosFlags.SWP_FRAMECHANGED);
        }

        private void cbAutoFit_CheckedChanged(object sender, EventArgs e)
        {
            PersistentSettings.AutoSize = cbAutoFit.Checked;
        }

        private void cbStartAutomatically_CheckedChanged(object sender, EventArgs e)
        {
            AutoStartManager.IsAutoStartConfigured = cbStartAutomatically.Checked;
        }

        private void llAbout_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (AboutBox1 abox = new AboutBox1())
            {
                abox.ShowDialog(this);
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.ApplicationExitCall)
                return;

            e.Cancel = true;
            HideAndShowNotifyIcon();
        }

        private void HideAndShowNotifyIcon()
        {
            this.Hide();
            Program.NotifyIcon.Visible = true;

            if (!PersistentSettings.IconHintShown)
            {
                Program.NotifyIcon.ShowBalloonTip(30000,
                    "EQ2FSW",
                    "EQ2FSW has been minimized to the notification area and will continue to run " +
                    "in the background. To exit EQ2FSW, right click this icon and choose \"Exit EQ2FSW\".",
                    ToolTipIcon.Info);
                PersistentSettings.IconHintShown = true;
            }
        }

        private void MainForm_Resize(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                HideAndShowNotifyIcon();
                this.WindowState = FormWindowState.Normal;
            }
        }
    }
}
