﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.ComponentModel;

namespace Eq2FullscreenWindowizer
{
    internal static class NativeMethods
    {
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool EnumWindows(EnumWindowsProc lpEnumFunc, IntPtr lParam);

        private delegate bool EnumWindowsProc(IntPtr hWnd, IntPtr lParam);

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        private static extern int GetWindowTextLength(IntPtr hWnd);

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern int GetWindowText(IntPtr hWnd, StringBuilder lpString, int nMaxCount);

        internal static string GetWindowText(IntPtr hWnd)
        {
            int wtLen = GetWindowTextLength(hWnd);
            StringBuilder sb = new StringBuilder(wtLen + 1);
            int res = GetWindowText(hWnd, sb, sb.Capacity);
            if (res == 0)
            {
                int werror = Marshal.GetLastWin32Error();
                if (werror != 0)
                    throw new Win32Exception(werror);
            }
            return sb.ToString();
        }

        internal static IEnumerable<IntPtr> EnumWindows()
        {
            List<IntPtr> windowList = new List<IntPtr>();

            EnumWindowsProc ewp = delegate(IntPtr hWnd, IntPtr lParam)
            {
                windowList.Add(hWnd);
                return true;
            };

            EnumWindows(ewp, IntPtr.Zero);
            return windowList;
        }
    }
}
