﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Win32;

namespace Eq2FullscreenWindowizer
{
    internal static class PersistentSettings
    {
        private const string RegKey = "HKEY_CURRENT_USER\\SOFTWARE\\Eq2FullScreenWindowizer";
        private const string RegValue_ScreenIndex = "ScreenIndex";
        private const string RegValue_AutoSize = "AutoSize";
        private const string RegValue_IconHintShown = "IconHintShown";

        private static T GetValue<T>(string valueName, T defaultValue)
        {
            try
            {
                return (T)Convert.ChangeType(Registry.GetValue(RegKey, valueName, defaultValue), typeof(T));
            }
            catch
            {
                return defaultValue;
            }
        }

        private static void SetValue<T>(string valueName, T value)
        {
            try
            {
                Registry.SetValue(RegKey, valueName, value);
            }
            catch { }
        }

        internal static int ScreenIndex
        {
            get { return GetValue<int>(RegValue_ScreenIndex, 0); }
            set { SetValue<int>(RegValue_ScreenIndex, value); }
        }

        internal static bool AutoSize
        {
            get { return GetValue<int>(RegValue_AutoSize, 1) == 0 ? false : true; }
            set { SetValue<int>(RegValue_AutoSize, value ? 1 : 0); }
        }

        internal static bool IconHintShown
        {
            get { return GetValue<int>(RegValue_IconHintShown, 0) == 0 ? false : true; }
            set { SetValue<int>(RegValue_IconHintShown, value ? 1 : 0); }
        }
    }
}
