﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Reflection;
using System.IO;

namespace Eq2FullscreenWindowizer
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            m_notifyIcon = new NotifyIcon();
            m_notifyIcon.Icon = GetApplicationIcon();
            m_notifyIcon.Text = "EQ2 Fullscreen Windowizer";
            m_notifyIcon.DoubleClick += new EventHandler(m_notifyIcon_DoubleClick);
            m_notifyIcon.ContextMenu = new ContextMenu();
            m_notifyIcon.ContextMenu.MenuItems.Add("Show EQ2FSW Main Window", new EventHandler(m_notifyIcon_DoubleClick)).DefaultItem = true;
            m_notifyIcon.ContextMenu.MenuItems.Add("Exit EQ2FSW", new EventHandler(mnuClose_Click));
            m_notifyIcon.Visible = false;

            m_mainForm = new MainForm();

            m_showInitialForm = true;
            foreach (string str in args)
            {
                if (str.Equals("/autostart", StringComparison.OrdinalIgnoreCase))
                {
                    m_notifyIcon.Visible = true;
                    m_showInitialForm = false;
                }
            }

            Application.Idle += InitialIdle;
            Application.Run();
        }

        private static void m_notifyIcon_DoubleClick(object sender, EventArgs e)
        {
            m_notifyIcon.Visible = false;
            m_mainForm.Show();
        }

        private static void mnuClose_Click(object sender, EventArgs e)
        {
            m_notifyIcon.Visible = false;
            Application.Exit();
        }

        private static bool m_showInitialForm = false;
        private static NotifyIcon m_notifyIcon;
        private static MainForm m_mainForm;

        internal static NotifyIcon NotifyIcon
        {
            get { return m_notifyIcon; }
        }

        private static void InitialIdle(object sender, EventArgs args)
        {
            Application.Idle -= InitialIdle;
            if (m_showInitialForm)
            {
                m_mainForm.Show();
            }
        }

        internal static Icon GetApplicationIcon()
        {
            try
            {
                string resName = null;
                Assembly asm = Assembly.GetExecutingAssembly();
                foreach (string tresName in asm.GetManifestResourceNames())
                {
                    if (tresName.EndsWith(".monitor.ico", StringComparison.OrdinalIgnoreCase))
                    {
                        resName = tresName;
                        break;
                    }
                }
                if (resName != null)
                {
                    using (Stream icoStream = asm.GetManifestResourceStream(resName))
                    {
                        return new Icon(icoStream);
                    }
                }
                return null;
            }
            catch 
            {
                return null;
            }
        }
    }
}
