﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace Eq2FullscreenWindowizer
{
    internal class ShellHook : NativeWindow, IDisposable
    {
        public ShellHook()
        {
            WM_SHELLHOOKMESSAGE = (int)RegisterWindowMessage("SHELLHOOK");
            this.CreateHandle(new CreateParams());
        }

        ~ShellHook()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (this.Handle != IntPtr.Zero)
            {
                this.ReleaseHandle();
                this.DestroyHandle();
            }
        }

        public override void CreateHandle(CreateParams cp)
        {
            base.CreateHandle(cp);
            RegisterShellHookWindow(this.Handle);
        }

        public override void ReleaseHandle()
        {
            DeregisterShellHookWindow(this.Handle);
            base.ReleaseHandle();
        }

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        private static extern uint RegisterWindowMessage(string lpString);

        [DllImport("user32")]
        private static extern bool RegisterShellHookWindow(IntPtr hWnd);

        [DllImport("user32")]
        private static extern bool DeregisterShellHookWindow(IntPtr hWnd);

        private int WM_SHELLHOOKMESSAGE;

        private enum ShellHookMessages
        {
            HSHELL_WINDOWCREATED = 1,
            HSHELL_WINDOWDESTROYED = 2,
            HSHELL_ACTIVATESHELLWINDOW = 3,
            HSHELL_WINDOWACTIVATED = 4,
            HSHELL_GETMINRECT = 5,
            HSHELL_REDRAW = 6,
            HSHELL_TASKMAN = 7,
            HSHELL_LANGUAGE = 8,
            HSHELL_ACCESSIBILITYSTATE = 11,
            HSHELL_FLASH = 32774
        }

        protected override void WndProc(ref Message m)
        {
            if (m.Msg == WM_SHELLHOOKMESSAGE)
            {
                ShellHookMessages shm = (ShellHookMessages)m.WParam.ToInt32();
                switch (shm)
                {
                    case ShellHookMessages.HSHELL_WINDOWCREATED:
                        OnWindowCreated(m.LParam);
                        break;
                    default:
                        break;
                }
            }
            base.WndProc(ref m);
        }

        private void OnWindowCreated(IntPtr hWnd)
        {
            EventHandler<HWndEventArgs> temp = WindowCreated;
            if (temp != null)
            {
                temp(this, new HWndEventArgs(hWnd));
            }
        }

        public event EventHandler<HWndEventArgs> WindowCreated;
    }

    internal class HWndEventArgs : EventArgs
    {
        internal HWndEventArgs(IntPtr hWnd)
        {
            m_hWnd = hWnd;
        }

        private readonly IntPtr m_hWnd;

        internal IntPtr Handle
        {
            get { return m_hWnd; }
        }
    }
}
